local flowImg = actor.LoadSprite("image/flow.png", { 64, 64 })

local defaultSize = { x = 64, y = 64 }
local defaultOffset = { x = -32, y = -32 }

local function EmitFlow()
	local color = nil
	local z = math.random()
	local width = 64 * z
	local dir = math.random() > 0.5 and -1 or 1
	local jitter = math.random(-width, width)
	local pos = { x = 500, y =  dir * 240 * z + jitter }
	local body = eapi.NewBody(gameWorld, pos)
	eapi.SetVel(body, { x = -500 * (1 + z), y = 0 })
	util.DelayedDestroy(body, 2 - z)

	local scale = 1 + 1 * z
	local size = vector.Scale(defaultSize, scale)
	local offset = vector.Scale(defaultOffset, scale)
	local tile = eapi.NewTile(body, offset, size, flowImg, z - 100)
	eapi.Animate(tile, eapi.ANIM_LOOP, 32 + 32 * z, math.random())
	if dir > 0 then
		color = { r = 0.0, g = 0.8 * z, b = 0.6 + 0.4 * z, a = 0.5 }
	else
		color = { r = 1.0 * z, g = 0.2 * z, b = 0.6 - 0.2 * z, a = 0.5 }
	end
	eapi.SetColor(tile, color)
end

local function Start()
	for i = 1, 10, 1 do EmitFlow() end
	eapi.AddTimer(staticBody, 0.01, Start)
end

local function Background(body, sprite, color, z, flip, height, offset)
	local size = { x = 1000, y = height }
	local offset = { x = -500, y = offset }
	local tile = eapi.NewTile(body, offset, size, sprite, z)
	eapi.SetColor(tile, color)
	eapi.FlipY(tile, flip)
	return tile
end

local color1 = { r = 0.0, g = 0.0, b = 0.4 }
Background(staticBody, util.white, color1, -1020, false, 600, -300)
local color2 = { r = 0.8, g = 0.2, b = 0.3 }
Background(staticBody, util.gradient, color2, -1010, true, 300, -300)
local color3 = { r = 0.0, g = 0.6, b = 0.8 }
Background(staticBody, util.gradient, color3, -1010, false, 300, 0)

local swirlImg = actor.LoadSprite("image/swirl.png", { 64, 64 })

local function Swirl()
	local size = { x = 32, y = 32 }
	local offset = { x = -16, y = -16 }
	local dstSize = { x = 64, y = 64 }
	local dstOffset = { x = -32, y = -32 }
	local velocity = { x = -400, y = 0 }
	local pos = { x = 400, y = -140 }
	local function Emit()
		local q = math.random()
		pos.y = -156 + q * 32
		local z = -99.2 - 0.05 * q
		local body = eapi.NewBody(gameWorld, pos)
		util.DelayedDestroy(body, 2)
		local tile = eapi.NewTile(body, offset, size, swirlImg, z)
		eapi.Animate(tile, eapi.ANIM_LOOP, 64, 0)
		velocity.x = -500 + 100 * q
		eapi.SetVel(body, velocity)
		eapi.AnimatePos(tile, eapi.ANIM_CLAMP, dstOffset, 0.5, 0)
		eapi.AnimateSize(tile, eapi.ANIM_CLAMP, dstSize, 0.5, 0)
		local color = { r = 1.0, g = 0.9 - 0.4 * q, b = 0.0, a = 0.0 }
		eapi.SetColor(tile, color)
		color.a = 0.8
		eapi.AnimateColor(tile, eapi.ANIM_CLAMP, color, 0.5, 0)		
	end
	local function EmitContinious()
		for i = 1, 2, 1 do Emit() end
		if not flow.noSwirl then
			eapi.AddTimer(staticBody, 0.01, EmitContinious)
		end
	end
	EmitContinious()
end

local srcPos = { x = -4, y = -4 }
local srcSize = { x = 8, y = 8 }
local dstPos = { x = -16, y = -16 }
local dstSize = { x = 32, y = 32 }

local lastEmit = 0

local function EmitDust(acc, angleVar, scale)
	local burstColors = explode.burstColors
	local angle = math.random(-angleVar, angleVar)
	local color = burstColors[math.random(1, #burstColors)]
	local pos = eapi.GetPos(player.obj.body)
	pos.x = pos.x + math.random(-10, 10) - 10
	pos.y = pos.y + math.random(-5, 5) - 20
	local z = -99.205 + 0.01 * math.random()
	return explode.EmitParticle(pos, acc, scale or 0.7, z, color, angle,
				    srcPos, srcSize, dstPos, dstSize)		
end

local feetDustAcc = { x = -100, y = 50 }
local jetpackDustAcc = { x = -100, y = -10 }

local function JetpackDust()
	for i = 1, 5, 1 do EmitDust(jetpackDustAcc, 10, 1.0) end
end

local function Dust()
	if player.obj.onGround then EmitDust(feetDustAcc, 20) end
	if player.obj.flying then JetpackDust() end
	eapi.AddTimer(staticBody, 0.02, Dust)
end

local fallDustAcc = { x = 0, y = 100 }
local fallDustGravity = { x = -200, y = -250 }

local function Puff()
	eapi.PlaySound(gameWorld, "sound/leaves.ogg", 0, 0.5)
	for i = 1, 100, 1 do
		local body = EmitDust(fallDustAcc, 45)
		eapi.SetAcc(body, fallDustGravity)		
	end
end

flow = {
	JetpackDust = JetpackDust,

	Start = Start,
	Swirl = Swirl,
	Dust = Dust,
	Puff = Puff,
}
return flow
