local LEFT   = 1
local RIGHT  = 2
local UP     = 3
local DOWN   = 4

actor.Cage("Blocker", 0.6, 0.5)
actor.SimpleCollide("Blocker", "Player", actor.Blocker)

local function GetDirection(moves)
	return { x = moves[RIGHT] - moves[LEFT], y = moves[UP] - moves[DOWN] }
end

local function SetSpeed(player, speed)
	if speed then player.speed = speed end
	eapi.SetVel(player.body, vector.Normalize(player.vel, player.speed))
end

local function DestroyText(obj, keyDown)
	if obj.text and keyDown then
		util.Map(eapi.Destroy, obj.text)
		player.Circular()
		obj.text = nil
	end
end

local function Move(player, axis)
	return function(keyDown)
		DestroyText(player, keyDown)
		player.moves[axis] = (keyDown and 1) or 0
		player.vel = GetDirection(player.moves)
		SetSpeed(player)
	end
end

local function Reset(keyDown)
	if keyDown then util.Goto("konfeti") end
end

local function EnableInput(player)
	input.Bind("Left", true, Move(player, LEFT))
	input.Bind("Right", true, Move(player, RIGHT))
	input.Bind("Up", true, Move(player, UP))
	input.Bind("Down", true, Move(player, DOWN))
	return player
end

local function Create()
	local obj = { speed = 300,
		      class = "Player",
		      vel = vector.null,
		      pos = vector.null,
		      moves = { 0, 0, 0, 0 },
		      bb = { l = -4, r = 4, b = -4, t = 4 } }
	local text = "\"Arrow Keys\" - Evade"
	ribbons.ShowHelpText(obj, text)
	local obj = actor.Create(obj)
	return EnableInput(obj)
end

player = {
	Create = Create,
}
return player
