dofile("script/konfeti-player.lua")
dofile("script/ribbons.lua")
dofile("script/explode.lua")
dofile("script/timing.lua")
dofile("script/bomb.lua")

local me = player.Create()

-- preload polar ship so that it does not mess up transition effect
local shipImg = actor.LoadSprite("image/ship.png", { 64, 64 })
local konfetiImg = actor.LoadSprite("image/konfeti.png", { 64, 64 })

local white = { r = 1, g = 1, b = 1, a = 1 }

local rainbowColors = {
	{ r = 1.00, g = 0.00, b = 0.00, a = 0.5 },
	{ r = 1.00, g = 0.50, b = 0.00, a = 0.5 },
	{ r = 1.00, g = 1.00, b = 0.00, a = 0.5 },
	{ r = 0.00, g = 1.00, b = 0.00, a = 0.5 },
	{ r = 0.00, g = 1.00, b = 1.00, a = 0.5 },
	{ r = 0.00, g = 0.00, b = 1.00, a = 0.5 },
	{ r = 0.44, g = 0.13, b = 0.38, a = 0.5 },
}

local index = 0
local VelFn = nil
local konfetiSpeed = 100

local function RandomVel()
	return vector.Rotate({ x = konfetiSpeed, y = 0 }, 360 * util.Random())
end

local spinAngle = 90
local function SpiralVel(pos)
	return vector.Rotate(vector.Normalize(pos, konfetiSpeed), spinAngle)
end

local function OneVel(pos)
	return oneVel
end

local fnTable = {
	OneVel,
	RandomVel,
	SpiralVel,
}

local dead = false
local evaded = 0

local function Change()
	spinAngle = 360 * util.Random()
	local base = { x = konfetiSpeed, y = 0 }
	oneVel = vector.Rotate(base, spinAngle)
	VelFn = fnTable[util.Random(1, #fnTable)]
	eapi.AddTimer(staticBody, 0.5, Change)
	dead = false
end
Change()

local colorStep = 0
local flicker = 1.0
local function KonfetiColor(body, tile, life)
	local colorPos = colorStep % #rainbowColors
	local index = math.floor(colorPos) + 1
	local color1 = rainbowColors[index]
	colorStep = colorStep + 0.001

	local color2 = rainbowColors[((index - 1) % #rainbowColors) + 1]
	local color = util.Mix(color1, color2, colorStep % 1)
	local duration = flicker * life
		    
	color.a = 0.0
	eapi.SetColor(tile, color)

	color.a = 1.0
	eapi.AnimateColor(tile, eapi.ANIM_REVERSE_LOOP, color, duration, 0)
end

local function KonfetiVelocity(pos, body, life)
	local vel = VelFn(pos)
	vel = vector.Scale(vel, 1 / life)
	eapi.SetVel(body, vel)
	return vel
end
	
local delay = 0.2
local function Rotate(tile, vel, oldVel)
	local oldAngle = vector.Angle(oldVel)
	local newAngle = vector.Angle(vel)
	local angleDiff = newAngle - oldAngle
	if angleDiff > 180 then
		newAngle = newAngle - 360
	end
	if angleDiff < -180 then
		newAngle = newAngle + 360
	end
	util.RotateTile(tile, 180 + oldAngle)
	local angle = vector.Radians(180 + newAngle)
	eapi.AnimateAngle(tile, eapi.ANIM_CLAMP, vector.null, angle, delay, 0)
end

local function MixVectors(v1, v2, q)
	return vector.Add(vector.Scale(v1, q), vector.Scale(v2, 1.0 - q))
end

local function PlayerVel(playerPos, pos, variation, oldVel)
	local diff = vector.Sub(playerPos, pos)
	local len = vector.Length(diff)
	if len > 8.0 then
		local vel = vector.Normalize(diff, 2 * konfetiSpeed)
		local factor = 1.0 / math.max(1.0, 0.02 * len)
		vel = vector.Rotate(vel, variation)
		vel = MixVectors(vel, oldVel, factor)
		return vel
	end
end

local store = { }
store[me] = me
me.angle = 0

local function Manipulate(body, tile)
	local violent = false
	local function Adjst()
		if dead and not violent then
			local pos = eapi.GetPos(body)
			local oldVel = eapi.GetVel(body)
			local playerPos = eapi.GetPos(me.body)
			local vel = vector.Sub(pos, playerPos)
			vel = vector.Normalize(vel, 400)
			Rotate(tile, vel, oldVel)
			eapi.SetVel(body, vel)
			violent = true
			return
		end

		local vel = nil
		local pos = eapi.GetPos(body)
		local oldVel = eapi.GetVel(body)
		local prevVel = oldVel
		for i, _ in pairs(store) do			
			local entry = store[i]
			local target = entry.pos
			vel = PlayerVel(target, pos, entry.angle, prevVel)
			if not vel then return end
			prevVel = vel
		end
		eapi.AddTimer(body, delay, Adjst)
		Rotate(tile, vel, oldVel)
		eapi.SetVel(body, vel)
	end
	Adjst()
end

local size = { x = 64, y = 64 }
local offset = { x = -32, y = -32 }

local function Make(pos)
	local body = eapi.NewBody(gameWorld, pos)
	local tile = eapi.NewTile(body, offset, size, konfetiImg, 0)

	local maxLife = konfetiSpeed / 100
	local life = maxLife * (0.5 + 0.5 * util.Random())
	local vel = KonfetiVelocity(pos, body, life)
	local angle = vector.Angle(vel)
	util.RotateTile(tile, 180 + angle)

	KonfetiColor(body, tile, life)
	util.DelayedDestroy(body, life * 2)
	eapi.Animate(tile, eapi.ANIM_LOOP, 32 / life, 0)
	Manipulate(body, tile)
end

local function Detractor(pos, life, vel)
	local obj = { pos = pos,
		      angle = 180,
		      class = "Hole",
		      velocity = vel,
		      bb = { l = -16, r = 16, b = -16, t = 16 } }
	obj = actor.Create(obj)
	local function Remove()
		actor.Delete(obj)
		store[obj] = nil
		evaded = evaded + 1
	end
	eapi.AddTimer(obj.body, life, Remove)
	store[obj] = obj
end

local amountBase = 1000
local function Emit()
	local amount = amountBase / konfetiSpeed
	for i, _ in pairs(store) do
		store[i].pos = eapi.GetPos(store[i].body)
	end
	for i = 1, amount, 1 do
		Make({ x = util.Random(-500, 500), 
		       y = util.Random(-340, 340) })
	end
	eapi.AddTimer(staticBody, 0.01, Emit)
end
Emit()

local mobSpeed = 4
local function PlayerVsHole(pShape, mShape)
	actor.DeleteShape(actor.store[mShape])
	if not dead then
		eapi.PlaySound(gameWorld, "sound/beat.ogg")
		timing.IncHits()
	end
	mobSpeed = 4
	evaded = 0
	dead = true
end

actor.SimpleCollide("Player", "Hole", PlayerVsHole)

local filterImg = actor.LoadSprite("image/noise.png", { 400, 240 })

local function Finalize()
	local tile = actor.FillScreen(filterImg, 500)
	eapi.Animate(tile, eapi.ANIM_LOOP, 64, 0)
	eapi.SetColor(tile, util.invisible)
	eapi.AnimateColor(tile, eapi.ANIM_CLAMP, util.Gray(1), 5, 0)
	eapi.PlaySound(gameWorld, "sound/static.ogg", 0, 1, 5)
	eapi.FadeMusic(5.0)
	local function Polar()
		util.Goto("polar")
	end
	eapi.AddTimer(staticBody, 5, Polar)
end

local function Circular()
	if evaded >= 6 then 
		return Finalize()
	elseif evaded > 0 then
		ribbons.Flash(1, evaded)
	end
	local angle = 360 * util.Random()
	local pos = vector.Rotate({ x = 450, y = 0 }, angle)
	local vel = vector.Sub(eapi.GetPos(me.body), pos)
	eapi.AddTimer(staticBody, 2 * mobSpeed + 0.1, Circular)
	Detractor(pos, 2 * mobSpeed, vector.Scale(vel, 2 / mobSpeed))
	mobSpeed = mobSpeed - 0.4
end
player.Circular = Circular

eapi.PlayMusic("sound/amb.ogg")
state.cheater = false
state.hits = 0
state.time = 0
timing.Progress(1)
