local function Spin(player, angle, dir)
	local start = dir > 0 and 0 or 31
	local last = 31 - start
	local this = eapi.GetFrame(player.tile)
	eapi.AnimateFrame(player.tile, eapi.ANIM_LOOP, start, last, this, 0.5)

	eapi.SetVel(player.body, { x = dir, y = dir })
	util.RotateTile(player.tile, angle)
	local start = vector.Radians(angle)
	eapi.AnimateAngle(player.tile, eapi.ANIM_LOOP, vector.null,
			  start + dir * 2 * math.pi, 2 * math.pi, 0)
end

local function ChangeDirection(player, keyDown)
	if keyDown and not player.dirKey then
		local burstAngle = -60 * player.dir
		local pos = eapi.GetPos(player.body, gameWorld)
		Spin(player, vector.Angle(pos) + 90, player.dir)
		explode.Burst(pos, vector.Rotate(pos, burstAngle), 60)
		eapi.PlaySound(gameWorld, "sound/woosh.ogg")
		player.dir = -player.dir
	end
	player.dirKey = keyDown
end

local function Move(player)
	return function(keyDown)
		ChangeDirection(player, keyDown)
	end
end

local function AddProgressCircle(player)
	local z = player.z + 1
	local size = { x = 128, y = 128 }
	local offset = { x = -64, y = -64 }
	player.circle = eapi.NewTile(player.body, offset, size, util.shine, z)
	util.AnimateRotation(player.circle, 8)
	eapi.SetColor(player.circle, { r = 1.0, g = 1.0, b = 0.0, a = 0.25 })
	local color = { r = 1.0, g = 0.5, b = 0.0, a = 0.5 }
	eapi.AnimateColor(player.circle, eapi.ANIM_REVERSE_LOOP, color, 0.1, 0)
end

local function FirstTap(player, Launch)
	return function(keyDown)
		if keyDown then
			AddProgressCircle(player)
			input.Bind("Move", false, Move(player))
			ChangeDirection(player, keyDown)
			util.Map(eapi.Destroy, player.text)
			bomb.ResetTime()
			Launch()
		end
	end
end

local function EnableInput(player, Launch)
	input.Bind("Move", false, FirstTap(player, Launch))
	return player
end

local shipImg = actor.LoadSprite("image/ship.png", { 64, 64 })

local function TrailTheTrail(obj)
	local ttl = 0.5
	local dstPos = { x = -64, y = -64 }
	local dstSize = { x = 128, y = 128 }
	local partialInvisibility = { a = 0.2 }
	local redInvisible = { r = 1.0, g = 0.2, b = 0.4, a = 0.0 }

	local function Emit()
		local pos = eapi.GetPos(obj.body)
		local body = eapi.NewBody(gameWorld, pos)
		local tile = eapi.NewTile(body, obj.offset, nil, obj.sprite, 9)
		eapi.SetFrame(tile, eapi.GetFrame(obj.tile))
		eapi.SetVel(body, vector.Scale(pos, 2))
		util.DelayedDestroy(body, ttl)
		eapi.SetColor(tile, partialInvisibility)
		eapi.AnimateColor(tile, eapi.ANIM_CLAMP, redInvisible, ttl, 0)
		eapi.AnimateSize(tile, eapi.ANIM_CLAMP, dstSize, ttl, 0)
		eapi.AnimatePos(tile, eapi.ANIM_CLAMP, dstPos, ttl, 0)
		local angle = vector.Angle(pos) + 90
		util.RotateTile(tile, angle)
		eapi.AddTimer(obj.body, 0.01, Emit)
	end
	Emit()
end

local function Create(Launch)
	local obj = { z = 10,
		      dir = 1,
		      speed = 300,
		      dirKey = false,
		      sprite = shipImg,
		      class = "Player",
		      vel = vector.null,
		      moves = { 0, 0, 0, 0 },
		      pos = { x = 0, y = -200 },
		      offset = { x = -32, y = -32 },
		      parentBody = eapi.NewBody(gameWorld, vector.null),
		      bb = actor.Square(8) }	
	obj = actor.Create(obj)
	eapi.SetStepC(obj.body, eapi.STEPFUNC_ROT, 0)
	eapi.PlayMusic("sound/music.ogg", nil, 1.0, 1.0)
	local text = "\"Space\" - Change Direction"
	ribbons.ShowHelpText(obj, text)
	eapi.SetFrame(obj.tile, 16)
	TrailTheTrail(obj)
	player.obj = obj
	return EnableInput(obj, Launch)
end

local size = { x = 8, y = 8 }
local offset = { x = -4, y = -4 }

local circlePos = { x = -64, y = -64 }
local circleSize = { x = 128, y = 128 }

local function ShrinkCircle()
	local tile = player.obj.circle
	local time = player.patternTime
	eapi.SetPos(tile, circlePos)
	eapi.SetSize(tile, circleSize)
	eapi.AnimatePos(tile, eapi.ANIM_CLAMP, offset, time, 0)
	eapi.AnimateSize(tile, eapi.ANIM_CLAMP, size, time, 0)
end

player = {
	Create = Create,
	patternTime = 6.28,
	ShrinkCircle = ShrinkCircle,
}
return player
