local cloudImg = actor.LoadSprite("image/cloud.png", { 128, 128 })

local tableSize = 100
local spaceTable = { }

local function Cloud(z, len, ttl, sprite, CallBack)
	local pos = { x = math.random(-400 - len, 400 + len), y = 240 + len }
	local body = eapi.NewBody(gameWorld, pos)
	local size = { x = 2 * len, y = 2 * len }
	local offset = { x = -len, y = -len }
	local tile = eapi.NewTile(body, offset, size, sprite, -100 + z)
	eapi.SetVel(body, { x = 0, y = -(480 + 2 * len) / ttl })
	util.MaybeCall(CallBack, body, size, offset)
	util.DelayedDestroy(body, ttl)
	eapi.FlipY(tile, true)
	return tile
end

local function FlowingNebula()
	local z = math.random()
	local tile = Cloud(z, 64 + 64 * z, 4 - 2 * z, cloudImg)
	eapi.Animate(tile, eapi.ANIM_LOOP, 32, 2 * math.random())
	local color = { r = 0.8 * z, g = 0.15 * z, b = 0.6 - 0.3 * z }
	color.a = 0.5 - 0.4 * z
	eapi.SetColor(tile, color)
end

local starColor = { r = 1.0, g = 1.0, b = 1.0, a = 0.1 }
local starDstColor = { r = 1.0, g = 1.0, b = 1.0, a = 0.5 }

local function FallingStars()
	local z = math.random()
	local tile = Cloud(z, 4 + 4 * z, 4 - 2 * z, explode.twinkleImg)
	eapi.SetColor(tile, starColor)
	eapi.AnimateColor(tile, eapi.ANIM_REVERSE_LOOP, starDstColor, 0.05, 0)
	explode.SetRandomAngle(tile)
end

local cloudLife = 1

local function ExpandingCloud(pos, size, offset, cloudColor, z)
	local body = eapi.NewBody(gameWorld, pos)
	local tile = eapi.NewTile(body, offset, size, cloudImg, -100 - z)
	eapi.Animate(tile, eapi.ANIM_LOOP, 32, 2 * math.random())
	local dstSize = { x = size.x * 8, y = size.y * 16 }
	local dstOffset = { x = offset.x * 8, y = offset.y * 16 }
	eapi.AnimateSize(tile, eapi.ANIM_CLAMP, dstSize, cloudLife, 0)
	eapi.AnimatePos(tile, eapi.ANIM_CLAMP, dstOffset, cloudLife, 0)
	util.DelayedDestroy(body, cloudLife)		
	eapi.FlipY(tile, true)
	cloudColor.a = 1.0
	eapi.SetColor(tile, cloudColor)
	cloudColor.a = 0.0
	eapi.AnimateColor(tile, eapi.ANIM_CLAMP, cloudColor, cloudLife, 0)
end

local function Background()
	local function Start()
		FlowingNebula()
		FallingStars()
		eapi.AddTimer(staticBody, 0.01, Start)
	end
	Start()
end

space = {
	comets = false,
	Background = Background,
}
return space
